@@ -1,4 +1,4 @@
-;;; haskell-commands.el --- Commands that can be run on the process
+;;; haskell-commands.el --- Commands that can be run on the process -*- lexical-binding: t -*-
 
 ;;; Commentary:
 
@@ -6,7 +6,8 @@
 ;;; specific commands such as show type signature, show info, haskell process
 ;;; commands and etc.
 
-;; Copyright (c) 2014 Chris Done. All rights reserved.
+;; Copyright © 2014 Chris Done.  All rights reserved.
+;;             2016 Arthur Fayzrakhmanov
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -25,19 +26,21 @@
 
 (require 'cl-lib)
 (require 'etags)
+(require 'haskell-mode)
 (require 'haskell-compat)
 (require 'haskell-process)
 (require 'haskell-font-lock)
 (require 'haskell-interactive-mode)
 (require 'haskell-session)
 (require 'haskell-presentation-mode)
+(require 'haskell-utils)
 (require 'highlight-uses-mode)
+(require 'haskell-cabal)
 
-
-(defvar haskell-utils-async-post-command-flag nil
-  "Non-nil means some commands were triggered during async function execution.")
-(make-variable-buffer-local 'haskell-utils-async-post-command-flag)
-
+(defcustom haskell-mode-stylish-haskell-path "stylish-haskell"
+  "Path to `stylish-haskell' executable."
+  :group 'haskell
+  :type 'string)
 
 ;;;###autoload
 (defun haskell-process-restart ()
@@ -65,7 +68,7 @@ You can create new session using function `haskell-session-make'."
     (haskell-process-set (haskell-session-process session) 'is-restarting nil)
     (let ((default-directory (haskell-session-cabal-dir session))
           (log-and-command (haskell-process-compute-process-log-and-command session (haskell-process-type))))
-      (haskell-session-pwd session)
+      (haskell-session-prompt-set-current-dir session (not haskell-process-load-or-reload-prompt))
       (haskell-process-set-process
        process
        (progn
@@ -74,7 +77,8 @@ You can create new session using function `haskell-session-make'."
     (progn (set-process-sentinel (haskell-process-process process) 'haskell-process-sentinel)
            (set-process-filter (haskell-process-process process) 'haskell-process-filter))
     (haskell-process-send-startup process)
-    (unless (eq 'cabal-repl (haskell-process-type)) ;; "cabal repl" sets the proper CWD
+    (unless (or (eq 'cabal-repl (haskell-process-type))
+                   (eq 'stack-ghci (haskell-process-type))) ;; Both "cabal repl" and "stack ghci" set the proper CWD.
       (haskell-process-change-dir session
                                   process
                                   (haskell-session-current-dir session)))
@@ -92,9 +96,17 @@ You can create new session using function `haskell-session-make'."
     :state process
 
     :go (lambda (process)
+          ;; We must set the prompt last, so that this command as a
+          ;; whole produces only one prompt marker as a response.
+          (haskell-process-send-string process
+                                       (mapconcat #'identity
+                                                  '("Prelude.putStrLn \"\""
+                                                    ":set -v1"
+                                                    ":set +c") ; :type-at in GHC 8+
+                                                  "\n"))
           (haskell-process-send-string process ":set prompt \"\\4\"")
-          (haskell-process-send-string process "Prelude.putStrLn \"\"")
-          (haskell-process-send-string process ":set -v1"))
+          (haskell-process-send-string process (format ":set prompt2 \"%s\""
+                                                       haskell-interactive-prompt2)))
 
     :live (lambda (process buffer)
             (when (haskell-process-consume
@@ -148,36 +160,25 @@ If I break, you can:
 Restores -fobject-code after reload finished.
 MODULE-BUFFER is the actual Emacs buffer of the module being loaded."
   (haskell-process-queue-without-filters process ":set -fbyte-code")
-  (haskell-process-touch-buffer process module-buffer)
-  (haskell-process-queue-without-filters process ":reload")
-  (haskell-process-queue-without-filters process ":set -fobject-code"))
-
-;;;###autoload
-(defun haskell-process-touch-buffer (process buffer)
-  "Query PROCESS to `:!touch` BUFFER's file.
-Use to update mtime on BUFFER's file."
-  (interactive)
-  (haskell-process-queue-command
+  ;; We prefix the module's filename with a "*", which asks ghci to
+  ;; ignore any existing object file and interpret the module.
+  ;; Dependencies will still use their object files as usual.
+  (haskell-process-queue-without-filters
    process
-   (make-haskell-command
-    :state (cons process buffer)
-    :go (lambda (state)
-          (haskell-process-send-string
-           (car state)
-           (format ":!%s %s"
-                   "touch"
-                   (shell-quote-argument (buffer-file-name
-                                          (cdr state))))))
-    :complete (lambda (state _)
-                (with-current-buffer (cdr state)
-                  (clear-visited-file-modtime))))))
+   (format ":load \"*%s\""
+           (replace-regexp-in-string
+            "\""
+            "\\\\\""
+            (buffer-file-name module-buffer))))
+  (haskell-process-queue-without-filters process ":set -fobject-code"))
 
 (defvar url-http-response-status)
 (defvar url-http-end-of-headers)
+(defvar haskell-cabal-targets-history nil
+  "History list for session targets.")
 
 (defun haskell-process-hayoo-ident (ident)
-  ;; FIXME Obsolete doc string, CALLBACK is not used.
-  "Hayoo for IDENT, return a list of modules asyncronously through CALLBACK."
+  "Hayoo for IDENT, return a list of modules"
   ;; We need a real/simulated closure, because otherwise these
   ;; variables will be unbound when the url-retrieve callback is
   ;; called.
@@ -288,7 +289,7 @@ Prompts for an arbitrary regexp given a prefix arg PROMPT."
                  (read-from-minibuffer "Look for: ")
                (haskell-ident-at-point))))
     (rgrep sym
-           "*.hs" ;; TODO: common Haskell extensions.
+           "*.hs *.lhs *.hsc *.chs *.hs-boot *.lhs-boot"
            (haskell-session-current-dir (haskell-interactive-session)))))
 
 ;;;###autoload
@@ -321,7 +322,6 @@ If PROMPT-VALUE is non-nil, request identifier via mini-buffer."
 
 ;;;###autoload
 (defun haskell-process-do-type (&optional insert-value)
-  ;; FIXME insert value functionallity seems to be missing.
   "Print the type of the given expression.
 
 Given INSERT-VALUE prefix indicates that result type signature
@@ -348,7 +348,7 @@ should be inserted."
           expr))))))
 
 ;;;###autoload
-(defun haskell-mode-jump-to-def-or-tag (&optional next-p)
+(defun haskell-mode-jump-to-def-or-tag (&optional _next-p)
   ;; FIXME NEXT-P arg is not used
   "Jump to the definition.
 Jump to definition of identifier at point by consulting GHCi, or
@@ -361,18 +361,13 @@ If the definition or tag is found, the location from which you jumped
 will be pushed onto `xref--marker-ring', so you can return to that
 position with `xref-pop-marker-stack'."
   (interactive "P")
-  (let ((initial-loc (point-marker))
-        (loc (haskell-mode-find-def (haskell-ident-at-point))))
-    (if loc
-        (haskell-mode-handle-generic-loc loc)
-      (call-interactively 'haskell-mode-tag-find))
-    (unless (equal initial-loc (point-marker))
-      (with-current-buffer (marker-buffer initial-loc)
-        (save-excursion
-          (goto-char initial-loc)
-          (set-mark-command nil)
-          ;; Store position for return with `xref-pop-marker-stack'
-          (xref-push-marker-stack))))))
+  (if (haskell-session-maybe)
+        (let ((initial-loc (point-marker))
+            (loc (haskell-mode-find-def (haskell-ident-at-point))))
+          (haskell-mode-handle-generic-loc loc)
+          (unless (equal initial-loc (point-marker))
+            (xref-push-marker-stack initial-loc)))
+      (call-interactively 'haskell-mode-tag-find)))
 
 ;;;###autoload
 (defun haskell-mode-goto-loc ()
@@ -393,7 +388,7 @@ Requires the :loc-at command from GHCi."
   (forward-char (plist-get span :start-col)))
 
 (defun haskell-process-insert-type ()
-  "Get the identifer at the point and insert its type.
+  "Get the identifier at the point and insert its type.
 Use GHCi's :type if it's possible."
   (let ((ident (haskell-ident-at-point)))
     (when ident
@@ -414,7 +409,7 @@ Use GHCi's :type if it's possible."
                        ;; TODO: Generalize this into a function.
                        ((or (string-match "^Top level" response)
                             (string-match "^<interactive>" response))
-                        (message response))
+                        (message "%s" response))
                        (t
                         (with-current-buffer (nth 2 state)
                           (goto-char (line-beginning-position))
@@ -473,7 +468,12 @@ Returns:
   "Either jump to or echo a generic location LOC.
 Either a file or a library."
   (cl-case (car loc)
-    (file (haskell-mode-jump-to-loc (cdr loc)))
+    (file (progn
+              (find-file (elt loc 1))
+              (goto-char (point-min))
+              (forward-line (1- (elt loc 2)))
+              (goto-char (+ (line-beginning-position)
+                            (1- (elt loc 3))))))
     (library (message "Defined in `%s' (%s)."
                       (elt loc 2)
                       (elt loc 1)))
@@ -518,12 +518,12 @@ Requires the :loc-at command from GHCi."
                              'face 'compilation-error)))))))
 
 ;;;###autoload
-(defun haskell-process-cd (&optional not-interactive)
+(defun haskell-process-cd (&optional _not-interactive)
   ;; FIXME optional arg is not used
   "Change directory."
   (interactive)
   (let* ((session (haskell-interactive-session))
-         (dir (haskell-session-pwd session t)))
+         (dir (haskell-session-prompt-set-current-dir session)))
     (haskell-process-log
      (propertize (format "Changing directory to %s ...\n" dir)
                  'face font-lock-comment-face))
@@ -531,23 +531,25 @@ Requires the :loc-at command from GHCi."
                                 (haskell-interactive-process)
                                 dir)))
 
-(defun haskell-session-pwd (session &optional change)
+(defun haskell-session-buffer-default-dir (session &optional buffer)
+  "Try to deduce a sensible default directory for SESSION and BUFFER,
+of which the latter defaults to the current buffer."
+  (or (haskell-session-get session 'current-dir)
+      (haskell-session-get session 'cabal-dir)
+      (if (buffer-file-name buffer)
+          (file-name-directory (buffer-file-name buffer))
+          "~/")))
+
+(defun haskell-session-prompt-set-current-dir (session &optional use-default)
   "Prompt for the current directory.
-Return current working directory for SESSION.
-Optional CHANGE argument makes user to choose new working directory for SESSION.
-In this case new working directory path will be returned."
-  (or (unless change
-        (haskell-session-get session 'current-dir))
-      (progn (haskell-session-set-current-dir
-              session
-              (haskell-utils-read-directory-name
-               (if change "Change directory: " "Set current directory: ")
-               (or (haskell-session-get session 'current-dir)
-                   (haskell-session-get session 'cabal-dir)
-                   (if (buffer-file-name)
-                       (file-name-directory (buffer-file-name))
-                     "~/"))))
-             (haskell-session-get session 'current-dir))))
+Return current working directory for SESSION."
+  (let ((default (haskell-session-buffer-default-dir session)))
+    (haskell-session-set-current-dir
+     session
+     (if use-default
+         default
+         (haskell-utils-read-directory-name "Set current directory: " default))))
+  (haskell-session-get session 'current-dir))
 
 (defun haskell-process-change-dir (session process dir)
   "Change SESSION's current directory.
@@ -588,7 +590,7 @@ Query PROCESS to `:cd` to directory DIR."
              (if (string-match "^[A-Za-z_]" (cdr state))
                  (format ":info %s" (cdr state))
                (format ":info (%s)" (cdr state)))))
-      :complete (lambda (state response)
+      :complete (lambda (_state response)
                   (unless (or (string-match "^Top level" response)
                               (string-match "^<interactive>" response))
                     (haskell-mode-message-line response)))))))
@@ -606,7 +608,7 @@ Query PROCESS to `:cd` to directory DIR."
              (if (string-match "^[A-Za-z_]" (cdr state))
                  (format ":type %s" (cdr state))
                (format ":type (%s)" (cdr state)))))
-      :complete (lambda (state response)
+      :complete (lambda (_state response)
                   (unless (or (string-match "^Top level" response)
                               (string-match "^<interactive>" response))
                     (haskell-mode-message-line response)))))))
@@ -614,20 +616,18 @@ Query PROCESS to `:cd` to directory DIR."
 ;;;###autoload
 (defun haskell-mode-show-type-at (&optional insert-value)
   "Show type of the thing at point or within active region asynchronously.
-This function requires GHCi-ng and `:set +c` option enabled by
-default (please follow GHCi-ng README available at URL
-`https://github.com/chrisdone/ghci-ng').
+This function requires GHCi 8+ or GHCi-ng.
 
 \\<haskell-interactive-mode-map>
 To make this function works sometimes you need to load the file in REPL
-first using command `haskell-process-load-or-reload' bound to
-\\[haskell-process-load-or-reload].
+first using command `haskell-process-load-file' bound to
+\\[haskell-process-load-file].
 
 Optional argument INSERT-VALUE indicates that
 recieved type signature should be inserted (but only if nothing
 happened since function invocation)."
   (interactive "P")
-  (let* ((pos (haskell-utils-capture-expr-bounds))
+  (let* ((pos (haskell-command-capture-expr-bounds))
          (req (haskell-utils-compose-type-at-command pos))
          (process (haskell-interactive-process))
          (buf (current-buffer))
@@ -651,23 +651,15 @@ happened since function invocation)."
                (min-pos (caar pos-reg))
                (max-pos (cdar pos-reg))
                (sig (haskell-utils-reduce-string response))
-               (res-type (haskell-utils-parse-repl-response sig)))
+               (res-type (haskell-utils-repl-response-error-status sig)))
 
           (cl-case res-type
             ;; neither popup presentation buffer
             ;; nor insert response in error case
             ('unknown-command
-             (message
-              (concat
-               "This command requires GHCi-ng. "
-               "Please read command description for details.")))
+             (message "This command requires GHCi 8+ or GHCi-ng. Please read command description for details."))
             ('option-missing
-             (message
-              (concat
-               "Could not infer type signature. "
-               "You need to load file first. "
-               "Also :set +c is required. "
-               "Please read command description for details.")))
+             (message "Could not infer type signature. You need to load file first. Also :set +c is required. Please read command description for details."))
             ('interactive-error (message "Wrong REPL response: %s" sig))
             (otherwise
              (if insert-value
@@ -682,22 +674,19 @@ happened since function invocation)."
                              (goto-char min-pos)
                              (insert (concat "(" sig ")"))))
                        ;; Non-region cases
-                       (haskell-utils-insert-type-signature sig))
+                       (haskell-command-insert-type-signature sig))
                    ;; Some commands registered, prevent insertion
-                   (let* ((rev (reverse haskell-utils-async-post-command-flag))
-                          (cs (format "%s" (cdr rev))))
-                     (message
-                      (concat
-                       "Type signature insertion was prevented. "
-                       "These commands were registered:"
-                       cs))))
+                   (message "Type signature insertion was prevented. These commands were registered: %s"
+                            (cdr (reverse haskell-utils-async-post-command-flag))))
                ;; Present the result only when response is valid and not asked
                ;; to insert result
-               (haskell-utils-echo-or-present response)))
+               (haskell-command-echo-or-present response)))
 
             (haskell-utils-async-stop-watching-changes init-buffer))))))))
 
-;;;###autoload
+(make-obsolete 'haskell-process-generate-tags
+               'haskell-mode-generate-tags
+               "2016-03-14")
 (defun haskell-process-generate-tags (&optional and-then-find-this-tag)
   "Regenerate the TAGS table.
 If optional AND-THEN-FIND-THIS-TAG argument is present it is used with
@@ -708,28 +697,19 @@ function `xref-find-definitions' after new table was generated."
      process
      (make-haskell-command
       :state (cons process and-then-find-this-tag)
-      :go (lambda (state)
-            (if (eq system-type 'windows-nt)
-                (haskell-process-send-string
-                 (car state)
-                 (format ":!hasktags --output=\"%s\\TAGS\" -x -e \"%s\""
-                            (haskell-session-cabal-dir (haskell-process-session (car state)))
-                            (haskell-session-cabal-dir (haskell-process-session (car state)))))
-              (haskell-process-send-string
-               (car state)
-               (format ":!cd %s && %s | %s"
-                       (haskell-session-cabal-dir
-                        (haskell-process-session (car state)))
-                       "find . -name '*.hs' -print0 -or -name '*.lhs' -print0 -or -name '*.hsc' -print0"
-                       "xargs -0 hasktags -e -x"))))
-      :complete (lambda (state response)
+      :go
+      (lambda (state)
+        (let* ((process (car state))
+               (cabal-dir (haskell-session-cabal-dir
+                           (haskell-process-session process)))
+               (command (haskell-cabal--compose-hasktags-command cabal-dir)))
+          (haskell-process-send-string process command)))
+      :complete (lambda (state _response)
                   (when (cdr state)
-                    (let ((session-tags
-                          (haskell-session-tags-filename
-                           (haskell-process-session (car state)))))
-                      (add-to-list 'tags-table-list session-tags)
-                      (setq tags-file-name nil))
-                    (xref-find-definitions (cdr state)))
+                    (let ((tags-file-name
+                           (haskell-session-tags-filename
+                            (haskell-process-session (car state)))))
+                      (xref-find-definitions (cdr state))))
                   (haskell-mode-message-line "Tags generated."))))))
 
 (defun haskell-process-add-cabal-autogen ()
@@ -754,94 +734,100 @@ inferior GHCi process."
   (interactive)
   (let ((session (haskell-interactive-session))
         (changed nil))
-    (if (null (haskell-session-get session
-                                   'ignored-files))
+    (if (null (haskell-session-get session 'ignored-files))
         (message "Nothing to unignore!")
-      (cl-loop for file in (haskell-session-get session
-                                                'ignored-files)
-               do (cl-case (read-event
-                            (propertize (format "Set permissions? %s (y, n, v: stop and view file)"
-                                                file)
-                                        'face 'minibuffer-prompt))
-                    (?y
-                     (haskell-process-unignore-file session file)
-                     (setq changed t))
-                    (?v
-                     (find-file file)
-                     (cl-return))))
-      (when (and changed
-                 (y-or-n-p "Restart GHCi process now? "))
-        (haskell-process-restart)))))
+      (cl-loop for file in (haskell-session-get session 'ignored-files)
+               do
+               (haskell-mode-toggle-interactive-prompt-state)
+               (unwind-protect
+                   (progn
+                     (cl-case
+                         (read-event
+                          (propertize
+                           (format "Set permissions? %s (y, n, v: stop and view file)"
+                                   file)
+                           'face
+                           'minibuffer-prompt))
+                       (?y
+                        (haskell-process-unignore-file session file)
+                        (setq changed t))
+                       (?v
+                        (find-file file)
+                        (cl-return)))
+                     (when (and changed
+                                (y-or-n-p "Restart GHCi process now? "))
+                       (haskell-process-restart)))
+                 ;; unwind
+                 (haskell-mode-toggle-interactive-prompt-state t))))))
 
 ;;;###autoload
 (defun haskell-session-change-target (target)
   "Set the build TARGET for cabal REPL."
-  (interactive "sNew build target:")
+  (interactive
+   (list
+    (completing-read "New build target: "
+                     (haskell-cabal-enum-targets)
+                     nil
+                     nil
+                     nil
+                     'haskell-cabal-targets-history)))
   (let* ((session haskell-session)
          (old-target (haskell-session-get session 'target)))
     (when session
       (haskell-session-set-target session target)
-      (when (and (not (string= old-target target))
-                 (y-or-n-p "Target changed, restart haskell process?"))
-        (haskell-process-start session)))))
+      (when (not (string= old-target target))
+        (haskell-mode-toggle-interactive-prompt-state)
+        (unwind-protect
+            (when (y-or-n-p "Target changed, restart haskell process?")
+              (haskell-process-start session)))
+        (haskell-mode-toggle-interactive-prompt-state t)))))
 
 ;;;###autoload
 (defun haskell-mode-stylish-buffer ()
-  "Apply stylish-haskell to the current buffer."
+  "Apply stylish-haskell to the current buffer.
+
+Use `haskell-mode-stylish-haskell-path' to know where to find
+stylish-haskell executable. This function tries to preserve
+cursor position and markers by using
+`haskell-mode-buffer-apply-command'."
   (interactive)
-  (let ((column (current-column))
-        (line (line-number-at-pos)))
-    (haskell-mode-buffer-apply-command "stylish-haskell")
-    (goto-char (point-min))
-    (forward-line (1- line))
-    (goto-char (+ column (point)))))
+  (haskell-mode-buffer-apply-command haskell-mode-stylish-haskell-path))
 
 (defun haskell-mode-buffer-apply-command (cmd)
   "Execute shell command CMD with current buffer as input and output.
 Use buffer as input and replace the whole buffer with the
 output.  If CMD fails the buffer remains unchanged."
   (set-buffer-modified-p t)
-  (let* ((chomp (lambda (str)
-                  (while (string-match "\\`\n+\\|^\\s-+\\|\\s-+$\\|\n+\\'" str)
-                    (setq str (replace-match "" t t str)))
-                  str))
-         (errout (lambda (fmt &rest args)
-                   (let* ((warning-fill-prefix "    "))
-                     (display-warning cmd (apply 'format fmt args) :warning))))
-         (filename (buffer-file-name (current-buffer)))
-         (cmd-prefix (replace-regexp-in-string " .*" "" cmd))
-         (tmp-file (make-temp-file cmd-prefix))
-         (err-file (make-temp-file cmd-prefix))
-         (default-directory (if (and (boundp 'haskell-session)
-                                     haskell-session)
-                                (haskell-session-cabal-dir haskell-session)
-                              default-directory))
-         (errcode (with-temp-file tmp-file
-                    (call-process cmd filename
-                                  (list (current-buffer) err-file) nil)))
-         (stderr-output
-          (with-temp-buffer
-            (insert-file-contents err-file)
-            (funcall chomp (buffer-substring-no-properties (point-min) (point-max)))))
-         (stdout-output
-          (with-temp-buffer
-            (insert-file-contents tmp-file)
-            (buffer-substring-no-properties (point-min) (point-max)))))
-    (if (string= "" stderr-output)
-        (if (string= "" stdout-output)
-            (message "Error: %s produced no output, leaving buffer alone" cmd)
-          (save-restriction
-            (widen)
-            ;; command successful, insert file with replacement to preserve
-            ;; markers.
-            (insert-file-contents tmp-file nil nil nil t)))
-      (progn
-        ;; non-null stderr, command must have failed
-        (message "Error: %s ended with errors, leaving buffer alone" cmd)
-        ;; use (warning-minimum-level :debug) to see this
-        (display-warning cmd stderr-output :debug)))
-    (delete-file tmp-file)
-    (delete-file err-file)))
+  (let* ((out-file (make-temp-file "stylish-output"))
+         (err-file (make-temp-file "stylish-error")))
+        (unwind-protect
+          (let* ((_errcode
+                  (call-process-region (point-min) (point-max) cmd nil
+                                       `((:file ,out-file) ,err-file)
+                                       nil))
+                 (err-file-empty-p
+                  (equal 0 (nth 7 (file-attributes err-file))))
+                 (out-file-empty-p
+                  (equal 0 (nth 7 (file-attributes out-file)))))
+            (if err-file-empty-p
+                (if out-file-empty-p
+                    (message "Error: %s produced no output and no error information, leaving buffer alone" cmd)
+                  ;; Command successful, insert file with replacement to preserve
+                  ;; markers.
+                  (insert-file-contents out-file nil nil nil t))
+              (progn
+                ;; non-null stderr, command must have failed
+                (message "Error: %s ended with errors, leaving buffer alone" cmd)
+                (with-temp-buffer
+                  (insert-file-contents err-file)
+                  ;; use (warning-minimum-level :debug) to see this
+                  (display-warning cmd
+                                   (buffer-substring-no-properties (point-min) (point-max))
+                                   :debug)))))
+          (ignore-errors
+            (delete-file err-file))
+          (ignore-errors
+            (delete-file out-file)))))
 
 ;;;###autoload
 (defun haskell-mode-find-uses ()
@@ -913,7 +899,17 @@ Requires the :uses command from GHCi."
           (error (propertize "No reply. Is :uses supported?"
                              'face 'compilation-error)))))))
 
-(defun haskell-utils-capture-expr-bounds ()
+(defun haskell-command-echo-or-present (msg)
+  "Present message in some manner depending on configuration.
+If variable `haskell-process-use-presentation-mode' is NIL it will output
+modified message MSG to echo area."
+  (if haskell-process-use-presentation-mode
+      (let ((session (haskell-process-session (haskell-interactive-process))))
+        (haskell-presentation-present session msg))
+    (let ((m (haskell-utils-reduce-string msg)))
+      (message "%s" m))))
+
+(defun haskell-command-capture-expr-bounds ()
   "Capture position bounds of expression at point.
 If there is an active region then it returns region
 bounds.  Otherwise it uses `haskell-spanable-pos-at-point` to
@@ -926,25 +922,7 @@ to point."
       (haskell-spanable-pos-at-point)
       (cons (point) (point))))
 
-(defun haskell-utils-compose-type-at-command (pos)
-  "Prepare :type-at command to be send to haskell process.
-POS is a cons cell containing min and max positions, i.e. target
-expression bounds."
-  (replace-regexp-in-string
-   "\n$"
-   ""
-   (format ":type-at %s %d %d %d %d %s"
-           (buffer-file-name)
-           (progn (goto-char (car pos))
-                  (line-number-at-pos))
-           (1+ (current-column))
-           (progn (goto-char (cdr pos))
-                  (line-number-at-pos))
-           (1+ (current-column))
-           (buffer-substring-no-properties (car pos)
-                                           (cdr pos)))))
-
-(defun haskell-utils-insert-type-signature (signature)
+(defun haskell-command-insert-type-signature (signature)
   "Insert type signature.
 In case of active region is present, wrap it by parentheses and
 append SIGNATURE to original expression.  Otherwise tries to
@@ -960,72 +938,5 @@ newlines and extra whitespace in signature before insertion."
         (insert sig "\n")
         (indent-to col)))))
 
-(defun haskell-utils-echo-or-present (msg)
-  "Present message in some manner depending on configuration.
-If variable `haskell-process-use-presentation-mode' is NIL it will output
-modified message MSG to echo area."
-  (if haskell-process-use-presentation-mode
-      (let ((session (haskell-process-session (haskell-interactive-process))))
-        (haskell-present session msg))
-    (let ((m (haskell-utils-reduce-string msg)))
-      (message m))))
-
-(defun haskell-utils-async-update-post-command-flag ()
-  "A special hook which collects triggered commands during async execution.
-This hook pushes value of variable `this-command' to flag variable
-`haskell-utils-async-post-command-flag'."
-  (let* ((cmd this-command)
-         (updated-flag (cons cmd haskell-utils-async-post-command-flag)))
-    (setq haskell-utils-async-post-command-flag updated-flag)))
-
-(defun haskell-utils-async-watch-changes ()
-  "Watch for triggered commands during async operation execution.
-Resets flag variable
-`haskell-utils-async-update-post-command-flag' to NIL.  By chanhges it is
-assumed that nothing happened, e.g. nothing was inserted in
-buffer, point was not moved, etc.  To collect data `post-command-hook' is used."
-  (setq haskell-utils-async-post-command-flag nil)
-  (add-hook
-   'post-command-hook #'haskell-utils-async-update-post-command-flag nil t))
-
-(defun haskell-utils-async-stop-watching-changes (buffer)
-  "Clean up after async operation finished.
-This function takes care about cleaning up things made by
-`haskell-utils-async-watch-changes'.  The BUFFER argument is a buffer where
-`post-command-hook' should be disabled.  This is neccessary, because
-it is possible that user will change buffer during async function
-execusion."
-  (with-current-buffer buffer
-    (setq haskell-utils-async-post-command-flag nil)
-    (remove-hook
-     'post-command-hook #'haskell-utils-async-update-post-command-flag t)))
-
-(defun haskell-utils-reduce-string (s)
-  "Remove newlines ans extra whitespace from S.
-Removes all extra whitespace at the beginning of each line leaving
-only single one.  Then removes all newlines."
-  (let ((s_ (replace-regexp-in-string "^\s+" " " s)))
-    (replace-regexp-in-string "\n" "" s_)))
-
-(defun haskell-utils-parse-repl-response (r)
-  "Parse response R from REPL and return special kind of result.
-The result is response string itself with speacial property
-response-type added.
-
-This property could be of the following:
-
-+ unknown-command
-+ option-missing
-+ interactive-error
-+ success"
-  (let ((first-line (car (split-string r "\n"))))
-    (cond
-     ((string-match-p "^unknown command" first-line) 'unknown-command)
-     ((string-match-p "^Couldn't guess that module name. Does it exist?"
-                      first-line)
-      'option-missing)
-     ((string-match-p "^<interactive>:" first-line) 'interactive-error)
-     (t 'success))))
-
 (provide 'haskell-commands)
 ;;; haskell-commands.el ends here
