@@ -1,4 +1,4 @@
-;;; haskell-compat.el --- legacy/compatibility backports for haskell-mode
+;;; haskell-compat.el --- legacy/compatibility backports for haskell-mode -*- lexical-binding: t -*-
 ;;
 ;; Filename: haskell-compat.el
 ;; Description: legacy/compatibility backports for haskell-mode
@@ -29,23 +29,15 @@
 (eval-when-compile
   (setq byte-compile-warnings '(not cl-functions obsolete)))
 
-;; Missing in Emacs23, stolen from Emacs24's `subr.el'
-(unless (fboundp 'process-live-p)
-  (defun process-live-p (process)
-    "Returns non-nil if PROCESS is alive.
-A process is considered alive if its status is `run', `open',
-`listen', `connect' or `stop'."
-    (memq (process-status process)
-	  '(run open listen connect stop))))
 
 ;; Cross-referencing commands have been replaced since Emacs 25.1.
 ;; These aliases are required to provide backward compatibility.
 (unless (fboundp 'xref-push-marker-stack)
   (defalias 'xref-pop-marker-stack 'pop-tag-mark)
 
-  (defun xref-push-marker-stack ()
-    "Add point to the marker stack."
-    (ring-insert find-tag-marker-ring (point-marker))))
+  (defun xref-push-marker-stack (&optional m)
+    "Add point M (defaults to `point-marker') to the marker stack."
+    (ring-insert find-tag-marker-ring (or m (point-marker)))))
 
 (unless (fboundp 'outline-hide-sublevels)
   (defalias 'outline-hide-sublevels 'hide-sublevels))
@@ -65,6 +57,9 @@ A process is considered alive if its status is `run', `open',
                        xref-prompt-for-identifier)))
       (find-tag ident next-p))))
 
+(unless (fboundp 'font-lock-ensure)
+  (defalias 'font-lock-ensure 'font-lock-fontify-buffer))
+
 (provide 'haskell-compat)
 
 ;;; haskell-compat.el ends here
