@@ -1,4 +1,4 @@
-;;; haskell-compile.el --- Haskell/GHC compilation sub-mode
+;;; haskell-compile.el --- Haskell/GHC compilation sub-mode -*- lexical-binding: t -*-
 
 ;; Copyright (C) 2013  Herbert Valerio Riedel
 
@@ -29,6 +29,7 @@
 (require 'compile)
 (require 'haskell-cabal)
 
+;;;###autoload
 (defgroup haskell-compile nil
   "Settings for Haskell compilation mode"
   :link '(custom-manual "(haskell-mode)compilation")
@@ -63,7 +64,7 @@ The `%s' placeholder is replaced by the current buffer's filename."
 
 (defconst haskell-compilation-error-regexp-alist
   `((,(concat
-       "^\\(?1:[^ \t\r\n]+?\\):"
+       "^ *\\(?1:[^\t\r\n]+?\\):"
        "\\(?:"
        "\\(?2:[0-9]+\\):\\(?4:[0-9]+\\)\\(?:-\\(?5:[0-9]+\\)\\)?" ;; "121:1" & "12:3-5"
        "\\|"
@@ -92,11 +93,9 @@ This is a child of `compilation-mode-map'.")
   "Local `compilation-filter-hook' for `haskell-compilation-mode'."
 
   (when haskell-compile-ghc-filter-linker-messages
-    (delete-matching-lines "^Loading package [^ \t\r\n]+ [.]+ linking [.]+ done\\.$"
-                           (if (boundp 'compilation-filter-start) ;; available since Emacs 24.2
-                               (save-excursion (goto-char compilation-filter-start)
-                                               (line-beginning-position))
-                             (point-min))
+    (delete-matching-lines "^ *Loading package [^ \t\r\n]+ [.]+ linking [.]+ done\\.$"
+                           (save-excursion (goto-char compilation-filter-start)
+                                           (line-beginning-position))
                            (point))))
 
 (define-compilation-mode haskell-compilation-mode "HsCompilation"
@@ -104,11 +103,9 @@ This is a child of `compilation-mode-map'.")
 This mode provides support for GHC 7.[46]'s compile
 messages. Specifically, also the `-ferror-spans` source location
 format is supported, as well as info-locations within compile
-messages pointing to additional source locations.
-
-See Info node `(haskell-mode)compilation' for more details."
-  (set (make-local-variable 'compilation-error-regexp-alist)
-       haskell-compilation-error-regexp-alist)
+messages pointing to additional source locations."
+  (setq-local compilation-error-regexp-alist
+              haskell-compilation-error-regexp-alist)
 
   (add-hook 'compilation-filter-hook
             'haskell-compilation-filter-hook nil t)
@@ -137,8 +134,7 @@ derived from `compilation-mode'. See Info
 node `(haskell-mode)compilation' for more details."
   (interactive "P")
   (save-some-buffers (not compilation-ask-about-save)
-                     (if (boundp 'compilation-save-buffers-predicate) ;; since Emacs 24.1(?)
-                         compilation-save-buffers-predicate))
+                         compilation-save-buffers-predicate)
   (let* ((cabdir (haskell-cabal-find-dir))
          (command1 (if (eq edit-command '-)
                        haskell-compile-cabal-build-alt-command
