@@ -1,7 +1,8 @@
-;;; haskell-doc.el --- show function types in echo area  -*- coding: utf-8 -*-
+;;; haskell-doc.el --- show function types in echo area  -*- coding: utf-8; lexical-binding: t -*-
 
-;; Copyright (C) 2004, 2005, 2006, 2007, 2009  Free Software Foundation, Inc.
-;; Copyright (C) 1997  Hans-Wolfgang Loidl
+;; Copyright © 2004, 2005, 2006, 2007, 2009, 2016  Free Software Foundation, Inc.
+;; Copyright © 1997  Hans-Wolfgang Loidl
+;;             2016  Arthur Fayzrakhmanov
 
 ;; Author: Hans-Wolfgang Loidl <hwloidl@dcs.glasgow.ac.uk>
 ;; Temporary Maintainer and Hacker: Graeme E Moss <gem@cs.york.ac.uk>
@@ -31,27 +32,14 @@
 ;; checking the word under the cursor and matching it against a list of
 ;; prelude, library, local and global functions.
 
-;; To show types of global functions, i.e. functions defined in a module
-;; imported by the current module, call the function
-;; `turn-on-haskell-doc-global-types'.  This automatically loads all modules
-;; and builds `imenu' tables to get the types of all functions.
-;; Note: The modules are loaded recursively, so you might pull in
-;;       many modules by just turning on global function support.
-;; This features is currently not very well supported.
-
 ;; This program was inspired by the `eldoc.el' package by Noah Friedman.
 
 ;; Installation:
 
-;; One useful way to enable this minor mode is to put the following in your
-;; .emacs:
+;; Depending on the major mode you use for your Haskell programs add
+;; one of the following to your .emacs:
 ;;
-;;      (autoload 'turn-on-haskell-doc-mode "haskell-doc" nil t)
-
-;;   and depending on the major mode you use for your Haskell programs:
-;;      (add-hook 'hugs-mode-hook 'turn-on-haskell-doc-mode)    ; hugs-mode
-;;     or
-;;      (add-hook 'haskell-mode-hook 'turn-on-haskell-doc-mode) ; haskell-mode
+;;   (add-hook 'haskell-mode-hook 'haskell-doc-mode)
 
 ;; Customisation:
 
@@ -84,7 +72,7 @@
 
 ;; `haskell-doc-mode' is implemented as a minor-mode.  So, you can combine it
 ;; with any other mode.  To enable it just type
-;;   M-x turn-on-haskell-doc-mode
+;;   M-x haskell-doc-mode
 
 ;; These are the names of the functions that can be called directly by the
 ;; user (with keybindings in `haskell-mode'):
@@ -118,6 +106,11 @@
 ;;; Changelog:
 
 ;;  $Log: haskell-doc.el,v $
+;;  Revision 1.31 2015/07/23 10:34:20  ankhers
+;;  (turn-on-haskell-doc-mode): marked obsolete
+;;  (turn-on-haskell-doc): marked obsolete
+;;  other packages have been moving away from (turn-on-haskell-*)
+;;
 ;;  Revision 1.30  2009/02/02 21:00:33  monnier
 ;;  (haskell-doc-imported-list): Don't add current buffer
 ;;  to the imported file list if it is not (yet?) visiting a file.
@@ -307,54 +300,22 @@
 
 ;;; Code:
 
-;;@menu
-;;* Constants and Variables::
-;;* Install as minor mode::
-;;* Menubar Support::
-;;* Haskell Doc Mode::
-;;* Switch it on or off::
-;;* Check::
-;;* Top level function::
-;;* Mouse interface::
-;;* Print fctsym::
-;;* Movement::
-;;* Bug Reports::
-;;* Visit home site::
-;;* Index::
-;;* Token::
-;;@end menu
-
-;;@node top, Constants and Variables, (dir), (dir)
-;;@top
-
-;;@node Constants and Variables, Install as minor mode, top, top
-;;@section Constants and Variables
-
-;;@menu
-;;* Emacs portability::
-;;* Maintenance stuff::
-;;* Mode Variable::
-;;* Variables::
-;;* Prelude types::
-;;* Test membership::
-;;@end menu
-
-;;@node Emacs portability, Maintenance stuff, Constants and Variables, Constants and Variables
-;;@subsection Emacs portability
-
 (require 'haskell-mode)
+(require 'haskell-process)
+(require 'haskell)
+(require 'haskell-utils)
 (require 'inf-haskell)
 (require 'imenu)
+(require 'eldoc)
 
+;;;###autoload
 (defgroup haskell-doc nil
   "Show Haskell function types in echo area."
   :group 'haskell
   :prefix "haskell-doc-")
 
-;;@node Mode Variable, Variables, Maintenance stuff, Constants and Variables
-;;@subsection Mode Variable
 
-(defvar haskell-doc-mode nil
+(defvar-local haskell-doc-mode nil
   "*If non-nil, show the type of the function near point or a related comment.
 
 If the identifier near point is a Haskell keyword and the variable
@@ -382,17 +343,15 @@ in the alist `haskell-doc-user-defined-ids' and the variable
 `haskell-doc-show-user-defined' is non-nil show the type of the function.
 
 This variable is buffer-local.")
-(make-variable-buffer-local 'haskell-doc-mode)
 
 (defvar haskell-doc-mode-hook nil
   "Hook invoked when entering `haskell-doc-mode'.")
 
-(defvar haskell-doc-index nil
+(defvar-local haskell-doc-index nil
   "Variable holding an alist matching file names to fct-type alists.
 The function `haskell-doc-make-global-fct-index' rebuilds this variables
 \(similar to an `imenu' rescan\).
 This variable is buffer-local.")
-(make-variable-buffer-local 'haskell-doc-index)
 
 (defcustom haskell-doc-show-global-types nil
   "If non-nil, search for the types of global functions by loading the files.
@@ -447,8 +406,6 @@ This variable is buffer-local."
 (defvar haskell-doc-search-distance 40  ; distance in characters
   "*How far to search when looking for the type declaration of fct under cursor.")
 
-;;@node Variables, Prelude types, Mode Variable, Constants and Variables
-;;@subsection Variables
 
 (defvar haskell-doc-idle-delay 0.50
   "*Number of seconds of idle time to wait before printing.
@@ -458,7 +415,7 @@ last input, no documentation will be printed.
 If this variable is set to 0, no idle time is required.")
 
 (defvar haskell-doc-argument-case 'identity ; 'upcase
-  "Case to display argument names of functions, as a symbol.
+  "Case in which to display argument names of functions, as a symbol.
 This has two preferred values: `upcase' or `downcase'.
 Actually, any name of a function which takes a string as an argument and
 returns another string is acceptable.")
@@ -502,11 +459,6 @@ It is probably best to manipulate this data structure with the commands
   "*String to display in mode line when Haskell-Doc Mode is enabled.")
 
 
-;;@node Prelude types, Test membership, Variables, Constants and Variables
-;;@subsection Prelude types
-
-;;@cindex haskell-doc-reserved-ids
-
 (defvar haskell-doc-reserved-ids
   '(("case" . "case exp of { alts [;] }")
     ("class" . "class [context =>] simpleclass [where { cbody [;] }]")
@@ -531,44 +483,13 @@ It is probably best to manipulate this data structure with the commands
     ("where" . "exp where { decl; ...; decl [;] }") ; check that ; see also class, instance, module
     ("as" . "import [qualified] modid [as modid] [impspec]")
     ("qualified" . "import [qualified] modid [as modid] [impspec]")
-    ("hiding" . "hiding ( import1 , ... , importn [ , ] )"))
+    ("hiding" . "hiding ( import1 , ... , importn [ , ] )")
+    ("family" . "(type family type [kind] [= type_fam_equations]) | (data family type [kind])"))
   "An alist of reserved identifiers.
 Each element is of the form (ID . DOC) where both ID and DOC are strings.
 DOC should be a concise single-line string describing the construct in which
 the keyword is used.")
 
-(eval-and-compile
-  (defalias 'haskell-doc-split-string
-    (if (condition-case ()
-            (split-string "" nil t)
-          (wrong-number-of-arguments nil))
-        'split-string
-      ;; copied from Emacs 22
-      (lambda (string &optional separators omit-nulls)
-        (let ((keep-nulls (not (if separators omit-nulls t)))
-              (rexp (or separators "[ \f\t\n\r\v]+"))
-              (start 0)
-              notfirst
-              (list nil))
-          (while (and (string-match rexp string
-                                    (if (and notfirst
-                                             (= start (match-beginning 0))
-                                             (< start (length string)))
-                                        (1+ start) start))
-                      (< start (length string)))
-            (setq notfirst t)
-            (if (or keep-nulls (< start (match-beginning 0)))
-                (setq list
-                      (cons (substring string start (match-beginning 0))
-                            list)))
-            (setq start (match-end 0)))
-          (if (or keep-nulls (< start (length string)))
-              (setq list
-                    (cons (substring string start)
-                          list)))
-          (nreverse list))))))
-
-;;@cindex haskell-doc-prelude-types
 
 (defun haskell-doc-extract-types (url)
   (with-temp-buffer
@@ -681,7 +602,7 @@ the keyword is used.")
                     ;;        module vars)
                     nil)
                 (setq curclass nil))
-              (dolist (var (haskell-doc-split-string vars comma-re t))
+              (dolist (var (split-string vars comma-re t))
                 (if (string-match "(.*)" var) (setq var (substring var 1 -1)))
                 (push (cons var type) elems))))
            ;; A datatype decl.
@@ -702,7 +623,7 @@ the keyword is used.")
                       (if (string-match ",\\'" type)
                           (setq type (substring type 0 -1)))
                       (setq type (concat name " -> " type))
-                      (dolist (var (haskell-doc-split-string vars comma-re t))
+                      (dolist (var (split-string vars comma-re t))
                         (if (string-match "(.*)" var)
                             (setq var (substring var 1 -1)))
                         (push (cons var type) elems))))))))
@@ -1223,7 +1144,6 @@ URL is the URL of the online doc."
     )
   "Alist of prelude functions and their types.")
 
-;;@cindex haskell-doc-strategy-ids
 
 (defvar haskell-doc-strategy-ids
   (list
@@ -1285,16 +1205,11 @@ URL is the URL of the online doc."
 (defvar haskell-doc-user-defined-ids nil
   "Alist of functions and strings defined by the user.")
 
-;;@node Test membership,  , Prelude types, Constants and Variables
-;;@subsection Test membership
 
-;;@cindex haskell-doc-is-of
 (defsubst haskell-doc-is-of (fn types)
   "Check whether FN is one of the functions in the alist TYPES and return the type."
   (assoc fn types) )
 
-;;@node Install as minor mode, Menubar Support, Constants and Variables, top
-;;@section Install as minor mode
 
 ;; Put this minor mode on the global minor-mode-alist.
 (or (assq 'haskell-doc-mode (default-value 'minor-mode-alist))
@@ -1302,13 +1217,6 @@ URL is the URL of the online doc."
                   (append (default-value 'minor-mode-alist)
                           '((haskell-doc-mode haskell-doc-minor-mode-string)))))
 
-
-;;@node Menubar Support, Haskell Doc Mode, Install as minor mode, top
-;;@section Menubar Support
-
-;; a dummy definition needed for XEmacs (I know, it's horrible :-(
-
-;;@cindex haskell-doc-install-keymap
 
 (defvar haskell-doc-keymap
   (let ((map (make-sparse-keymap)))
@@ -1352,11 +1260,6 @@ URL is the URL of the online doc."
   (local-set-key [(control shift meta mouse-3)]
                  'haskell-doc-ask-mouse-for-type))
 
-
-;;@node Haskell Doc Mode, Switch it on or off, Menubar Support, top
-;;@section Haskell Doc Mode
-
-;;@cindex haskell-doc-mode
 
 (defvar haskell-doc-timer nil)
 (defvar haskell-doc-buffers nil)
@@ -1419,7 +1322,6 @@ See variable docstring."
              (>= (prefix-numeric-value ,prefix) 0)
            (not ,id))) )
 
-;;@cindex haskell-doc-show-global-types
 (defun haskell-doc-show-global-types (&optional prefix)
   "Turn on global types information in `haskell-doc-mode'."
   (interactive "P")
@@ -1427,53 +1329,45 @@ See variable docstring."
   (if haskell-doc-show-global-types
       (haskell-doc-make-global-fct-index)))
 
-;;@cindex haskell-doc-show-reserved
 (defun haskell-doc-show-reserved (&optional prefix)
   "Toggle the automatic display of a doc string for reserved ids."
   (interactive "P")
   (haskell-doc-toggle-var haskell-doc-show-reserved prefix))
 
-;;@cindex haskell-doc-show-prelude
 (defun haskell-doc-show-prelude (&optional prefix)
   "Toggle the automatic display of a doc string for reserved ids."
   (interactive "P")
   (haskell-doc-toggle-var haskell-doc-show-prelude prefix))
 
-;;@cindex haskell-doc-show-strategy
 (defun haskell-doc-show-strategy (&optional prefix)
   "Toggle the automatic display of a doc string for strategy ids."
   (interactive "P")
   (haskell-doc-toggle-var haskell-doc-show-strategy prefix))
 
-;;@cindex haskell-doc-show-user-defined
 (defun haskell-doc-show-user-defined (&optional prefix)
   "Toggle the automatic display of a doc string for user defined ids."
   (interactive "P")
   (haskell-doc-toggle-var haskell-doc-show-user-defined prefix))
 
-;;@node Switch it on or off, Check, Haskell Doc Mode, top
-;;@section Switch it on or off
-
-;;@cindex turn-on-haskell-doc-mode
 
 ;;;###autoload
 (defalias 'turn-on-haskell-doc-mode 'haskell-doc-mode)
+(make-obsolete 'turn-on-haskell-doc-mode
+               'haskell-doc-mode
+               "2015-07-23")
 
 ;;;###autoload
 (defalias 'turn-on-haskell-doc 'haskell-doc-mode)
+(make-obsolete 'turn-on-haskell-doc
+               'haskell-doc-mode
+               "2015-07-23")
 
-;;@cindex  turn-off-haskell-doc-mode
 (defalias 'turn-off-haskell-doc-mode 'turn-off-haskell-doc)
 
 (defun turn-off-haskell-doc ()
   "Unequivocally turn off `haskell-doc-mode' (which see)."
   (haskell-doc-mode 0))
 
-;;@node Check, Top level function, Switch it on or off, top
-;;@section Check
-
-;;@cindex haskell-doc-check-active
-
 (defun haskell-doc-check-active ()
   "Check whether the print function is hooked in.
 Should be the same as the value of `haskell-doc-mode' but alas currently it
@@ -1487,10 +1381,7 @@ is not."
              (substitute-command-keys
               "haskell-doc is not ACTIVE \(Use \\[haskell-doc-mode] to turn it on\)"))))
 
-;;@node Top level function, Mouse interface, Check, top
-;;@section Top level function
 
-;;@cindex haskell-doc-mode-print-current-symbol-info
 ;; This is the function hooked into the elisp command engine
 (defun haskell-doc-mode-print-current-symbol-info ()
   "Print the type of the symbol under the cursor.
@@ -1498,11 +1389,14 @@ is not."
 This function is run by an idle timer to print the type
  automatically if `haskell-doc-mode' is turned on."
   (and haskell-doc-mode
+       (haskell-doc-in-code-p)
+       (not haskell-mode-interactive-prompt-state)
        (not (eobp))
        (not executing-kbd-macro)
        ;; Having this mode operate in the minibuffer makes it impossible to
        ;; see what you're doing.
        (not (eq (selected-window) (minibuffer-window)))
+       ;; not in string or comment
        ;; take a nap, if run straight from post-command-hook.
        (if (fboundp 'run-with-idle-timer) t
          (sit-for haskell-doc-idle-delay))
@@ -1519,13 +1413,14 @@ This function is run by an idle timer to print the type
 (defun haskell-doc-current-info ()
   "Return the info about symbol at point.
 Meant for `eldoc-documentation-function'."
-  (haskell-doc-sym-doc (haskell-ident-at-point)))
+  ;; There are a number of possible documentation functions.
+  ;; Some of them are asynchronous.
+  (when (haskell-doc-in-code-p)
+    (let ((msg (or
+                (haskell-doc-current-info--interaction)
+                (haskell-doc-sym-doc (haskell-ident-at-point)))))
+      (unless (symbolp msg) msg))))
 
-
-;;@node Mouse interface, Print fctsym, Top level function, top
-;;@section Mouse interface for interactive query
-
-;;@cindex haskell-doc-ask-mouse-for-type
 (defun haskell-doc-ask-mouse-for-type (event)
   "Read the identifier under the mouse and echo its type.
 This uses the same underlying function `haskell-doc-show-type' as the hooked
@@ -1536,23 +1431,8 @@ function.  Only the user interface is different."
     (goto-char (posn-point (event-end event)))
     (haskell-doc-show-type)))
 
-
-;;@node Print fctsym, Movement, Mouse interface, top
-;;@section Print fctsym
-
-;;@menu
-;;* Show type::
-;;* Aux::
-;;* Global fct type::
-;;* Local fct type::
-;;@end menu
-
-;;@node Show type, Aux, Print fctsym, Print fctsym
-;;@subsection Show type
-
-;;@cindex haskell-doc-show-type
-
 (defun haskell-doc-in-code-p ()
+  "A predicate indicating suitable case to show docs."
   (not (or (and (eq haskell-literate 'bird)
                 ;; Copied from haskell-indent-bolp.
                 (<= (current-column) 2)
@@ -1561,7 +1441,7 @@ function.  Only the user interface is different."
 
 ;;;###autoload
 (defun haskell-doc-show-type (&optional sym)
-  "Show the type of the function near point.
+  "Show the type of the function near point or given symbol SYM.
 For the function under point, show the type in the echo area.
 This information is extracted from the `haskell-doc-prelude-types' alist
 of prelude functions and their types, or from the local functions in the
@@ -1570,22 +1450,102 @@ current buffer."
   (unless sym (setq sym (haskell-ident-at-point)))
   ;; if printed before do not print it again
   (unless (string= sym (car haskell-doc-last-data))
-    (let ((doc (haskell-doc-sym-doc sym)))
+    (let ((doc (or (haskell-doc-current-info--interaction t)
+                  (haskell-doc-sym-doc sym))))
       (when (and doc (haskell-doc-in-code-p))
         ;; In Emacs 19.29 and later, and XEmacs 19.13 and later, all
         ;; messages are recorded in a log.  Do not put haskell-doc messages
         ;; in that log since they are legion.
-        (if (eval-when-compile (fboundp 'display-message))
-            ;; XEmacs 19.13 way of preventing log messages.
-            ;;(display-message 'no-log (format <args>))
-            ;; XEmacs 19.15 seems to be a bit different.
-            (display-message 'message (format "%s" doc))
-          (let ((message-log-max nil))
-            (message "%s" doc)))))))
+        (let ((message-log-max nil))
+          (message "%s" doc))))))
 
+(defvar haskell-doc-current-info--interaction-last nil
+  "Async message stack.
+If non-nil, a previous eldoc message from an async call, that
+hasn't been displayed yet.")
+
+(defun haskell-doc-current-info--interaction (&optional sync)
+  "Asynchronous call to `haskell-process-get-type'.
+Suitable for use in the eldoc function `haskell-doc-current-info'.
+
+If SYNC is non-nil, the call will be synchronous instead, and
+instead of calling `eldoc-print-current-symbol-info', the result
+will be returned directly."
+  ;; Return nil if nothing is available, or 'async if something might
+  ;; be available, but asynchronously later. This will call
+  ;; `eldoc-print-current-symbol-info' later.
+  (when (haskell-doc-in-code-p)
+    ;; do nothing when inside string or comment
+    (let (sym prev-message)
+      (cond
+       ((setq prev-message haskell-doc-current-info--interaction-last)
+        (setq haskell-doc-current-info--interaction-last nil)
+        (cdr prev-message))
+       ((setq sym
+              (if (use-region-p)
+                  (buffer-substring-no-properties
+                   (region-beginning) (region-end))
+                (haskell-ident-at-point)))
+        (if sync
+            (haskell-process-get-type sym #'identity t)
+          (haskell-process-get-type
+           sym (lambda (response)
+                 (setq haskell-doc-current-info--interaction-last
+                       (cons 'async response))
+                 (eldoc-print-current-symbol-info)))))))))
+
+(defun haskell-process-get-type (expr-string &optional callback sync)
+  "Asynchronously get the type of a given string.
+
+EXPR-STRING should be an expression passed to :type in ghci.
+
+CALLBACK will be called with a formatted type string.
+
+If SYNC is non-nil, make the call synchronously instead."
+  (unless callback (setq callback (lambda (response) (message "%s" response))))
+  (let ((process (and (haskell-session-maybe)
+                    (haskell-session-process (haskell-session-maybe))))
+        ;; Avoid passing bad strings to ghci
+        (expr-okay
+         (and (not (string-match-p "\\`[[:space:]]*\\'" expr-string))
+            (not (string-match-p "\n" expr-string))))
+        (ghci-command (concat ":type " expr-string))
+        (process-response
+         (lambda (response)
+           ;; Responses with empty first line are likely errors
+           (if (string-match-p (rx string-start line-end) response)
+               (setq response nil)
+             ;; Remove a newline at the end
+             (setq response (replace-regexp-in-string "\n\\'" "" response))
+             ;; Propertize for eldoc
+             (save-match-data
+               (when (string-match " :: " response)
+                 ;; Highlight type
+                 (let ((name (substring response 0 (match-end 0)))
+                       (type (propertize
+                              (substring response (match-end 0))
+                              'face 'eldoc-highlight-function-argument)))
+                   (setq response (concat name type)))))
+             (when haskell-doc-prettify-types
+               (dolist (re '(("::" . "∷") ("=>" . "⇒") ("->" . "→")))
+                 (setq response
+                       (replace-regexp-in-string (car re) (cdr re) response))))
+             response))))
+    (when (and process expr-okay)
+      (if sync
+          (let ((response (haskell-process-queue-sync-request process ghci-command)))
+            (funcall callback (funcall process-response response)))
+        (haskell-process-queue-command
+         process
+         (make-haskell-command
+          :go (lambda (_) (haskell-process-send-string process ghci-command))
+          :complete
+          (lambda (_ response)
+            (funcall callback (funcall process-response response)))))
+        'async))))
 
 (defun haskell-doc-sym-doc (sym)
-  "Show the type of the function near point.
+  "Show the type of given symbol SYM.
 For the function under point, show the type in the echo area.
 This information is extracted from the `haskell-doc-prelude-types' alist
 of prelude functions and their types, or from the local functions in the
@@ -1594,7 +1554,7 @@ If `haskell-doc-use-inf-haskell' is non-nil, this function will consult
 the inferior Haskell process for type/kind information, rather than using
 the haskell-doc database."
   (if haskell-doc-use-inf-haskell
-      (unless (string= "" sym)
+      (unless (or (null sym) (string= "" sym))
         (let* ((message-log-max nil)
                (result (ignore-errors
                          (unwind-protect
@@ -1612,8 +1572,7 @@ the haskell-doc database."
           (is-reserved (haskell-doc-is-of sym haskell-doc-reserved-ids))
           (is-prelude  (haskell-doc-is-of sym haskell-doc-prelude-types))
           (is-strategy (haskell-doc-is-of sym haskell-doc-strategy-ids))
-          (is-user-defined (haskell-doc-is-of sym haskell-doc-user-defined-ids))
-          (is-prelude  (haskell-doc-is-of sym haskell-doc-prelude-types)))
+          (is-user-defined (haskell-doc-is-of sym haskell-doc-user-defined-ids)))
       (cond
        ;; if reserved id (i.e. Haskell keyword
        ((and haskell-doc-show-reserved
@@ -1686,12 +1645,9 @@ the haskell-doc database."
 ;;    (and (symbolp sym)
 ;;         sym)))
 
-;;@node Aux, Global fct type, Show type, Print fctsym
-;;@subsection Aux
 
 ;; ToDo: handle open brackets to decide if it's a wrapped type
 
-;;@cindex haskell-doc-grab-line
 (defun haskell-doc-grab-line (fct-and-pos)
   "Get the type of an \(FCT POSITION\) pair from the current buffer."
   ;; (if (null fct-and-pos)
@@ -1714,7 +1670,6 @@ the haskell-doc database."
          str)))))
 ;; (concat (car fct-and-pos) "::" (haskell-doc-string-nub-ws str))))
 
-;;@cindex haskell-doc-wrapped-type-p
 (defun haskell-doc-wrapped-type-p ()
   "Check whether the type under the cursor is wrapped over several lines.
 The cursor must be at the end of a line, which contains the type.
@@ -1760,7 +1715,6 @@ ToDo: Check for matching parenthesis!."
                   (setq res t))))))
       res)))
 
-;;@cindex haskell-doc-grab
 (defun haskell-doc-grab ()
   "Return the text from point to the end of the line, chopping off comments.
 Leaves point at end of line."
@@ -1770,7 +1724,6 @@ Leaves point at end of line."
         (substring str 0 (match-beginning 0))
       str)))
 
-;;@cindex haskell-doc-string-nub-ws
 (defun haskell-doc-string-nub-ws (str)
   "Replace all sequences of whitespace in STR by just one space.
 ToDo: Also eliminate leading and trailing whitespace."
@@ -1779,24 +1732,6 @@ ToDo: Also eliminate leading and trailing whitespace."
       (setq str (replace-match " " t t str)))
     str))
 
-;; ToDo: make this more efficient!!
-;;(defun haskell-doc-string-nub-ws (str)
-;;  "Replace all sequences of whitespaces in STR by just one whitespace."
-;;  (let ( (res "")
-;;       (l (length str))
-;;       (i 0)
-;;       (j 0)
-;;       (in-ws nil))
-;;   (while (< i l)
-;;     (let* ( (c (string-to-char (substring str i (1+ i))))
-;;          (is-ws (eq (char-syntax c) ? )) )
-;;       (if (not (and in-ws is-ws))
-;;           (setq res (concat res (char-to-string c))))
-;;       (setq in-ws is-ws)
-;;       (setq i (1+ i))))
-;;   res))
-
-;;@cindex haskell-doc-chop-off-context
 (defun haskell-doc-chop-off-context (str)
   "Eliminate the context in a type represented by the string STR."
   (let ((i (string-match "=>" str)) )
@@ -1804,7 +1739,6 @@ ToDo: Also eliminate leading and trailing whitespace."
         str
       (substring str (+ i 2)))))
 
-;;@cindex haskell-doc-get-imenu-info
 (defun haskell-doc-get-imenu-info (obj kind)
   "Return a string describing OBJ of KIND \(Variables, Types, Data\)."
   (cond
@@ -1817,14 +1751,10 @@ ToDo: Also eliminate leading and trailing whitespace."
    (t ;; (error "Cannot get local functions in %s mode, sorry" major-mode)))
     nil)))
 
-;;@node Global fct type, Local fct type, Aux, Print fctsym
-;;@subsection Global fct type
-
 ;; ToDo:
 ;;  - modular way of defining a mapping of module name to file
 ;;  - use a path to search for file (not just current directory)
 
-;;@cindex haskell-doc-imported-list
 
 (defun haskell-doc-imported-list ()
   "Return a list of the imported modules in current buffer."
@@ -1846,8 +1776,6 @@ ToDo: Also eliminate leading and trailing whitespace."
 
 ;; ToDo: generalise this to "Types" etc (not just "Variables")
 
-;;@cindex haskell-doc-rescan-files
-
 (defun haskell-doc-rescan-files (filelist)
   "Do an `imenu' rescan on every file in FILELIST and return the fct-list.
 This function switches to and potentially loads many buffers."
@@ -1861,8 +1789,6 @@ This function switches to and potentially loads many buffers."
                             (cdr (assoc "Variables" imenu--index-alist)))))
             filelist)))
 
-;;@cindex haskell-doc-make-global-fct-index
-
 (defun haskell-doc-make-global-fct-index ()
   "Scan imported files for types of global fcts and update `haskell-doc-index'."
   (interactive)
@@ -1871,8 +1797,6 @@ This function switches to and potentially loads many buffers."
 
 ;; ToDo: use a separate munge-type function to format type concisely
 
-;;@cindex haskell-doc-get-global-fct-type
-
 (defun haskell-doc-get-global-fct-type (&optional sym)
   "Get type for function symbol SYM by examining `haskell-doc-index'."
   (interactive) ;  "fName of outer `include' file: \nsFct:")
@@ -1901,11 +1825,6 @@ This function switches to and potentially loads many buffers."
           (setq fal (cdr fal))))
       res))) ; (message res)) )
 
-;;@node Local fct type,  , Global fct type, Print fctsym
-;;@subsection Local fct type
-
-;;@cindex haskell-doc-get-and-format-fct-type
-
 (defun haskell-doc-get-and-format-fct-type (fn)
   "Get the type and kind of FN by checking local and global functions."
   (save-excursion
@@ -1929,48 +1848,6 @@ This function switches to and potentially loads many buffers."
         ;; return the result
         doc ))))
 
-
-;;@appendix
-
-;;@node Index, Token, Visit home site, top
-;;@section Index
-
-;;@index
-;;* haskell-doc-ask-mouse-for-type::
-;;* haskell-doc-check-active::
-;;* haskell-doc-chop-off-context::
-;;* haskell-doc-get-and-format-fct-type::
-;;* haskell-doc-get-global-fct-type::
-;;* haskell-doc-get-imenu-info::
-;;* haskell-doc-grab::
-;;* haskell-doc-grab-line::
-;;* haskell-doc-imported-list::
-;;* haskell-doc-install-keymap::
-;;* haskell-doc-is-of::
-;;* haskell-doc-make-global-fct-index::
-;;* haskell-doc-mode::
-;;* haskell-doc-mode-print-current-symbol-info::
-;;* haskell-doc-prelude-types::
-;;* haskell-doc-rescan-files::
-;;* haskell-doc-reserved-ids::
-;;* haskell-doc-show-global-types::
-;;* haskell-doc-show-prelude::
-;;* haskell-doc-show-reserved::
-;;* haskell-doc-show-strategy::
-;;* haskell-doc-show-type::
-;;* haskell-doc-show-user-defined::
-;;* haskell-doc-strategy-ids::
-;;* haskell-doc-string-nub-ws::
-;;* haskell-doc-submit-bug-report::
-;;* haskell-doc-visit-home::
-;;* haskell-doc-wrapped-type-p::
-;;* turn-off-haskell-doc-mode::
-;;* turn-on-haskell-doc-mode::
-;;@end index
-
-;;@node Token,  , Index, top
-;;@section Token
-
 (provide 'haskell-doc)
 
 ;;; haskell-doc.el ends here
