@@ -1,4 +1,4 @@
-;;; haskell-indent.el --- "semi-intelligent" indentation module for Haskell Mode
+;;; haskell-indent.el --- "semi-intelligent" indentation module for Haskell Mode -*- lexical-binding: t -*-
 
 ;; Copyright 2004, 2005, 2007, 2008, 2009  Free Software Foundation, Inc.
 ;; Copyright 1997-1998  Guy Lapalme
@@ -92,6 +92,7 @@
 
 (defvar haskell-literate)
 
+;;;###autoload
 (defgroup haskell-indent nil
   "Haskell indentation."
   :group 'haskell
@@ -322,7 +323,6 @@ It deals with both Bird style and non Bird-style scripts."
         (insert "\\begin{code}\n")))))
 
 ;;; Start of indentation code
-
 (defcustom haskell-indent-look-past-empty-line t
   "If nil, indentation engine will not look past an empty line for layout points."
   :group 'haskell-indent
@@ -436,7 +436,7 @@ Returns the location of the start of the comment, nil otherwise."
 
 (defun haskell-indent-next-symbol-safe (end)
   "Puts point to the next following symbol, or to end if there are no more symbols in the sexp."
-  (condition-case errlist (haskell-indent-next-symbol end)
+  (condition-case _errlist (haskell-indent-next-symbol end)
     (error (goto-char end))))
 
 (defun haskell-indent-separate-valdef (start end)
@@ -658,8 +658,8 @@ Returns the location of the start of the comment, nil otherwise."
           (string-match "where[ \t]*" haskell-indent-current-line-first-ident))
          (diff-first                 ; not a function def with the same name
           (or (null valname-string)
-              (not (string= (haskell-trim valname-string)
-                            (haskell-trim haskell-indent-current-line-first-ident)))))
+              (not (string= (haskell-string-trim valname-string)
+                            (haskell-string-trim haskell-indent-current-line-first-ident)))))
 
          ;; (is-type-def
          ;;  (and rhs-sign (eq (char-after rhs-sign) ?\:)))
@@ -1295,7 +1295,7 @@ We stay in the cycle as long as the TAB key is pressed."
         (if marker
             (goto-char (marker-position marker)))))))
 
-(defun haskell-indent-region (start end)
+(defun haskell-indent-region (_start _end)
   (error "Auto-reindentation of a region is not supported"))
 
 ;;; alignment functions
@@ -1433,7 +1433,7 @@ TYPE is either 'guard or 'rhs."
             (if regstack
                 (haskell-indent-shift-columns maxcol regstack)))))))
 
-(defun haskell-indent-align-guards-and-rhs (start end)
+(defun haskell-indent-align-guards-and-rhs (_start _end)
   "Align the guards and rhs of functions in the region, which must be active."
   ;; The `start' and `end' args are dummys right now: they're just there so
   ;; we can use the "r" interactive spec which properly signals an error.
@@ -1489,9 +1489,8 @@ One indentation cycle is used."
 
 ;;; haskell-indent-mode
 
-(defvar haskell-indent-mode nil
+(defvar-local haskell-indent-mode nil
   "Non-nil if the semi-intelligent Haskell indentation mode is in effect.")
-(make-variable-buffer-local 'haskell-indent-mode)
 
 (defvar haskell-indent-map
   (let ((map (make-sparse-keymap)))
@@ -1499,30 +1498,33 @@ One indentation cycle is used."
     ;; (define-key map "\177"  'backward-delete-char-untabify)
     ;; The binding to TAB is already handled by indent-line-function.  --Stef
     ;; (define-key map "\t"    'haskell-indent-cycle)
-    (define-key map [?\C-c ?\C-=] 'haskell-indent-insert-equal)
-    (define-key map [?\C-c ?\C-|] 'haskell-indent-insert-guard)
+    (define-key map (kbd "C-c C-=") 'haskell-indent-insert-equal)
+    (define-key map (kbd "C-c C-|") 'haskell-indent-insert-guard)
     ;; Alternate binding, in case C-c C-| is too inconvenient to type.
     ;; Duh, C-g is a special key, let's not use it here.
-    ;; (define-key map [?\C-c ?\C-g] 'haskell-indent-insert-guard)
-    (define-key map [?\C-c ?\C-o] 'haskell-indent-insert-otherwise)
-    (define-key map [?\C-c ?\C-w] 'haskell-indent-insert-where)
-    (define-key map [?\C-c ?\C-.] 'haskell-indent-align-guards-and-rhs)
-    (define-key map [?\C-c ?\C->] 'haskell-indent-put-region-in-literate)
+    ;; (define-key map (kbd "C-c C-g") 'haskell-indent-insert-guard)
+    (define-key map (kbd "C-c C-o") 'haskell-indent-insert-otherwise)
+    (define-key map (kbd "C-c C-w") 'haskell-indent-insert-where)
+    (define-key map (kbd "C-c C-.") 'haskell-indent-align-guards-and-rhs)
+    (define-key map (kbd "C-c C->") 'haskell-indent-put-region-in-literate)
     map))
 
 ;;;###autoload
 (defun turn-on-haskell-indent ()
   "Turn on ``intelligent'' Haskell indentation mode."
-  (set (make-local-variable 'indent-line-function) 'haskell-indent-cycle)
-  (set (make-local-variable 'indent-region-function) 'haskell-indent-region)
+  (when (and (bound-and-true-p haskell-indentation-mode)
+             (fboundp 'haskell-indentation-mode))
+    (haskell-indentation-mode 0))
+
+  (setq-local indent-line-function 'haskell-indent-cycle)
+  (setq-local indent-region-function 'haskell-indent-region)
   (setq haskell-indent-mode t)
   ;; Activate our keymap.
   (let ((map (current-local-map)))
     (while (and map (not (eq map haskell-indent-map)))
       (setq map (keymap-parent map)))
-    (if map
-        ;; haskell-indent-map is already active: nothing to do.
-        nil
+    ;; if haskell-indent-map is already active: there's nothing to do.
+    (unless map
       ;; Put our keymap on top of the others.  We could also put it in
       ;; second place, or in a minor-mode.  The minor-mode approach would be
       ;; easier, but it's harder for the user to override it.  This approach
@@ -1538,6 +1540,7 @@ One indentation cycle is used."
 (defun turn-off-haskell-indent ()
   "Turn off ``intelligent'' Haskell indentation mode."
   (kill-local-variable 'indent-line-function)
+  (kill-local-variable 'indent-region-function)
   ;; Remove haskell-indent-map from the local map.
   (let ((map (current-local-map)))
     (while map
@@ -1576,6 +1579,9 @@ these functions also align the guards and rhs of the current definition
     \\[haskell-indent-put-region-in-literate]
       makes the region a piece of literate code in a literate script
 
+If `ARG' is falsey, toggle `haskell-indent-mode'.  Else sets
+`haskell-indent-mode' to whether `ARG' is greater than 0.
+
 Invokes `haskell-indent-hook' if not nil."
   (interactive "P")
   (setq haskell-indent-mode
