@@ -1,6 +1,7 @@
-;;; haskell.el --- Top-level Haskell package
+;;; haskell.el --- Top-level Haskell package -*- lexical-binding: t -*-
 
-;; Copyright (c) 2014 Chris Done. All rights reserved.
+;; Copyright © 2014 Chris Done.  All rights reserved.
+;;             2016 Arthur Fayzrakhmanov
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -15,10 +16,13 @@
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+;;; Commentary:
+
 ;;; Code:
 
 (require 'cl-lib)
 (require 'haskell-mode)
+(require 'haskell-hoogle)
 (require 'haskell-process)
 (require 'haskell-debug)
 (require 'haskell-interactive-mode)
@@ -28,38 +32,45 @@
 (require 'haskell-sandbox)
 (require 'haskell-modules)
 (require 'haskell-string)
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Basic configuration hooks
-
-(add-hook 'haskell-process-ended-hook 'haskell-process-prompt-restart)
-(add-hook 'kill-buffer-hook 'haskell-interactive-kill)
+(require 'haskell-completions)
+(require 'haskell-utils)
+(require 'haskell-customize)
 
 (defvar interactive-haskell-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "C-c C-l") 'haskell-process-load-or-reload)
+    (define-key map (kbd "C-c C-l") 'haskell-process-load-file)
+    (define-key map (kbd "C-c C-r") 'haskell-process-reload)
     (define-key map (kbd "C-c C-t") 'haskell-process-do-type)
     (define-key map (kbd "C-c C-i") 'haskell-process-do-info)
     (define-key map (kbd "M-.") 'haskell-mode-jump-to-def-or-tag)
     (define-key map (kbd "C-c C-k") 'haskell-interactive-mode-clear)
     (define-key map (kbd "C-c C-c") 'haskell-process-cabal-build)
+    (define-key map (kbd "C-c v c") 'haskell-cabal-visit-file)
     (define-key map (kbd "C-c C-x") 'haskell-process-cabal)
-    (define-key map [?\C-c ?\C-b] 'haskell-interactive-switch)
-    (define-key map [?\C-c ?\C-z] 'haskell-interactive-switch)
+    (define-key map (kbd "C-c C-b") 'haskell-interactive-switch)
+    (define-key map (kbd "C-c C-z") 'haskell-interactive-switch)
     map)
-  "Keymap for using haskell-interactive-mode.")
+  "Keymap for using `interactive-haskell-mode'.")
 
 ;;;###autoload
 (define-minor-mode interactive-haskell-mode
   "Minor mode for enabling haskell-process interaction."
   :lighter " Interactive"
   :keymap interactive-haskell-mode-map
-  (add-hook 'completion-at-point-functions 'haskell-process-completions-at-point nil t))
+  (add-hook 'completion-at-point-functions
+            #'haskell-completions-sync-repl-completion-at-point
+            nil
+            t))
+
+(make-obsolete 'haskell-process-completions-at-point
+               'haskell-completions-sync-repl-completion-at-point
+               "June 19, 2015")
 
 (defun haskell-process-completions-at-point ()
-  "A completion-at-point function using the current haskell process."
+  "A `completion-at-point' function using the current haskell process."
   (when (haskell-session-maybe)
-    (let ((process (haskell-process)) symbol symbol-bounds)
+    (let ((process (haskell-process))
+          symbol-bounds)
       (cond
        ;; ghci can complete module names, but it needs the "import "
        ;; string at the beginning
@@ -74,7 +85,8 @@
               (end (match-end 1)))
           (list start end
                 (haskell-process-get-repl-completions process text))))
-       ;; Complete OPTIONS using :complete repl ":set ..."
+       ;; Complete OPTIONS, a completion list comes from variable
+       ;; `haskell-ghc-supported-options'
        ((and (nth 4 (syntax-ppss))
            (save-excursion
              (let ((p (point)))
@@ -84,7 +96,8 @@
             (rx symbol-start "-" (* (char alnum ?-)))
             (line-beginning-position)))
         (list (match-beginning 0) (match-end 0) haskell-ghc-supported-options))
-       ;; Complete LANGUAGE :complete repl ":set -X..."
+       ;; Complete LANGUAGE, a list of completions comes from variable
+       ;; `haskell-ghc-supported-extensions'
        ((and (nth 4 (syntax-ppss))
            (save-excursion
              (let ((p (point)))
@@ -104,50 +117,66 @@
   "Handle the return key."
   (interactive)
   (cond
+   ;; At a compile message, jump to the location of the error in the
+   ;; source.
    ((haskell-interactive-at-compile-message)
     (next-error-internal))
+   ;; At the input prompt, handle the expression in the usual way.
+   ((haskell-interactive-at-prompt)
+    (haskell-interactive-handle-expr))
+   ;; At any other location in the buffer, copy the line to the
+   ;; current prompt.
    (t
-    (haskell-interactive-handle-expr))))
+    (haskell-interactive-copy-to-prompt))))
 
 ;;;###autoload
 (defun haskell-session-kill (&optional leave-interactive-buffer)
   "Kill the session process and buffer, delete the session.
 0. Prompt to kill all associated buffers.
 1. Kill the process.
-2. Kill the interactive buffer.
+2. Kill the interactive buffer unless LEAVE-INTERACTIVE-BUFFER is not given.
 3. Walk through all the related buffers and set their haskell-session to nil.
 4. Remove the session from the sessions list."
   (interactive)
-  (let* ((session (haskell-session))
-         (name (haskell-session-name session))
-         (also-kill-buffers
-          (and haskell-ask-also-kill-buffers
-               (y-or-n-p (format "Killing `%s'. Also kill all associated buffers?" name)))))
-    (haskell-kill-session-process session)
-    (unless leave-interactive-buffer
-      (kill-buffer (haskell-session-interactive-buffer session)))
-    (cl-loop for buffer in (buffer-list)
-             do (with-current-buffer buffer
-                  (when (and (boundp 'haskell-session)
-                             (string= (haskell-session-name haskell-session) name))
-                    (setq haskell-session nil)
-                    (when also-kill-buffers
-                      (kill-buffer)))))
-    (setq haskell-sessions
-          (cl-remove-if (lambda (session)
-                          (string= (haskell-session-name session)
-                                   name))
-                        haskell-sessions))))
+  (haskell-mode-toggle-interactive-prompt-state)
+  (unwind-protect
+      (let* ((session (haskell-session))
+             (name (haskell-session-name session))
+             (also-kill-buffers
+              (and haskell-ask-also-kill-buffers
+                   (y-or-n-p
+                    (format "Killing `%s'. Also kill all associated buffers?"
+                            name)))))
+        (haskell-kill-session-process session)
+        (unless leave-interactive-buffer
+          (kill-buffer (haskell-session-interactive-buffer session)))
+        (cl-loop for buffer in (buffer-list)
+                 do (with-current-buffer buffer
+                      (when (and (boundp 'haskell-session)
+                                 (string= (haskell-session-name haskell-session)
+                                          name))
+                        (setq haskell-session nil)
+                        (when also-kill-buffers
+                          (kill-buffer)))))
+        (setq haskell-sessions
+              (cl-remove-if (lambda (session)
+                              (string= (haskell-session-name session)
+                                       name))
+                            haskell-sessions)))
+    (haskell-mode-toggle-interactive-prompt-state t)))
 
 ;;;###autoload
 (defun haskell-interactive-kill ()
   "Kill the buffer and (maybe) the session."
   (interactive)
   (when (eq major-mode 'haskell-interactive-mode)
-    (when (and (boundp 'haskell-session)
-               haskell-session
-               (y-or-n-p "Kill the whole session?"))
-      (haskell-session-kill t))))
+    (haskell-mode-toggle-interactive-prompt-state)
+    (unwind-protect
+        (when (and (boundp 'haskell-session)
+                   haskell-session
+                   (y-or-n-p "Kill the whole session?"))
+          (haskell-session-kill t)))
+    (haskell-mode-toggle-interactive-prompt-state t)))
 
 (defun haskell-session-make (name)
   "Make a Haskell session."
@@ -160,12 +189,16 @@
     session))
 
 (defun haskell-session-new-assume-from-cabal ()
-  "Prompt to create a new project based on a guess from the nearest Cabal file."
+  "Prompt to create a new project based on a guess from the nearest Cabal file.
+If `haskell-process-load-or-reload-prompt' is nil, accept `default'."
   (let ((name (haskell-session-default-name)))
     (unless (haskell-session-lookup name)
-      (when (y-or-n-p (format "Start a new project named “%s”? "
-                              name))
-        (haskell-session-make name)))))
+      (haskell-mode-toggle-interactive-prompt-state)
+      (unwind-protect
+          (if (or (not haskell-process-load-or-reload-prompt)
+                  (y-or-n-p (format "Start a new project named “%s”? " name)))
+              (haskell-session-make name))
+        (haskell-mode-toggle-interactive-prompt-state t)))))
 
 ;;;###autoload
 (defun haskell-session ()
@@ -193,10 +226,15 @@
   (let ((name (read-from-minibuffer "Project name: " (haskell-session-default-name))))
     (when (not (string= name ""))
       (let ((session (haskell-session-lookup name)))
-        (if session
-            (when (y-or-n-p (format "Session %s already exists. Use it?" name))
-              session)
-          (haskell-session-make name))))))
+        (haskell-mode-toggle-interactive-prompt-state)
+        (unwind-protect
+            (if session
+                (when
+                    (y-or-n-p
+                     (format "Session %s already exists. Use it?" name))
+                  session)
+              (haskell-session-make name)))
+        (haskell-mode-toggle-interactive-prompt-state t)))))
 
 ;;;###autoload
 (defun haskell-session-change ()
@@ -207,44 +245,62 @@
                               (haskell-session-new))))
 
 (defun haskell-process-prompt-restart (process)
-  "Prompt to restart the died process."
-  (let ((process-name (haskell-process-name process)))
+  "Prompt to restart the died PROCESS."
+  (let ((process-name (haskell-process-name process))
+        (cursor-in-echo-area t))
     (if haskell-process-suggest-restart
-        (cond
-         ((string-match "You need to re-run the 'configure' command."
-                        (haskell-process-response process))
-          (cl-case (read-event
-                    (concat "The Haskell process ended. Cabal wants you to run "
-                            (propertize "cabal configure" 'face 'font-lock-keyword-face)
-                            " because there is a version mismatch. Re-configure (y, n, l: view log)?"
-                            "\n\n"
-                            "Cabal said:\n\n"
-                            (propertize (haskell-process-response process)
-                                        'face 'font-lock-comment-face)))
-            (?y (let ((default-directory (haskell-session-cabal-dir (haskell-process-session process))))
-                  (message "%s" (shell-command-to-string "cabal configure"))))
-            (?l (let* ((response (haskell-process-response process))
-                       (buffer (get-buffer "*haskell-process-log*")))
-                  (if buffer
-                      (switch-to-buffer buffer)
-                    (progn (switch-to-buffer (get-buffer-create "*haskell-process-log*"))
-                           (insert response)))))
-            (?n)))
-         (t
-          (cl-case (read-event
-                    (propertize (format "The Haskell process `%s' has died. Restart? (y, n, l: show process log)"
-                                        process-name)
-                                'face 'minibuffer-prompt))
-            (?y (haskell-process-start (haskell-process-session process)))
-            (?l (let* ((response (haskell-process-response process))
-                       (buffer (get-buffer "*haskell-process-log*")))
-                  (if buffer
-                      (switch-to-buffer buffer)
-                    (progn (switch-to-buffer (get-buffer-create "*haskell-process-log*"))
-                           (insert response)))))
-            (?n))))
-      (message (format "The Haskell process `%s' is dearly departed."
-                       process-name)))))
+        (progn
+          (haskell-mode-toggle-interactive-prompt-state)
+          (unwind-protect
+              (cond
+               ((string-match "You need to re-run the 'configure' command."
+                              (haskell-process-response process))
+                (cl-case (read-char-choice
+                          (concat
+                           "The Haskell process ended. Cabal wants you to run "
+                           (propertize "cabal configure"
+                                       'face
+                                       'font-lock-keyword-face)
+                           " because there is a version mismatch. Re-configure (y, n, l: view log)?"
+                           "\n\n"
+                           "Cabal said:\n\n"
+                           (propertize (haskell-process-response process)
+                                       'face
+                                       'font-lock-comment-face))
+                          '(?l ?n ?y))
+                  (?y (let ((default-directory
+                              (haskell-session-cabal-dir
+                               (haskell-process-session process))))
+                        (message "%s"
+                                 (shell-command-to-string "cabal configure"))))
+                  (?l (let* ((response (haskell-process-response process))
+                             (buffer (get-buffer "*haskell-process-log*")))
+                        (if buffer
+                            (switch-to-buffer buffer)
+                          (progn (switch-to-buffer
+                                  (get-buffer-create "*haskell-process-log*"))
+                                 (insert response)))))
+                  (?n)))
+               (t
+                (cl-case (read-char-choice
+                          (propertize
+                           (format "The Haskell process `%s' has died. Restart? (y, n, l: show process log) "
+                                   process-name)
+                           'face
+                           'minibuffer-prompt)
+                          '(?l ?n ?y))
+                  (?y (haskell-process-start (haskell-process-session process)))
+                  (?l (let* ((response (haskell-process-response process))
+                             (buffer (get-buffer "*haskell-process-log*")))
+                        (if buffer
+                            (switch-to-buffer buffer)
+                          (progn (switch-to-buffer
+                                  (get-buffer-create "*haskell-process-log*"))
+                                 (insert response)))))
+                  (?n))))
+            ;; unwind
+            (haskell-mode-toggle-interactive-prompt-state t)))
+      (message "The Haskell process `%s' is dearly departed." process-name))))
 
 (defun haskell-process ()
   "Get the current process from the current session."
@@ -278,60 +334,35 @@
              (haskell-interactive-mode-error-backward)
              (haskell-interactive-jump-to-error-line)))))
 
-;;;###autoload
-(defun haskell-mode-contextual-space ()
-  "Contextually do clever stuff when hitting space."
-  (interactive)
-  (if (or (not (bound-and-true-p interactive-haskell-mode))
-          (not (haskell-session-maybe)))
-      (self-insert-command 1)
-    (cond ((and haskell-mode-contextual-import-completion
-                (save-excursion (forward-word -1)
-                                (looking-at "^import$")))
-           (insert " ")
-           (let ((module (haskell-complete-module-read
-                          "Module: "
-                          (haskell-session-all-modules (haskell-session)))))
-             (let ((mapping (assoc module haskell-import-mapping)))
-               (if mapping
-                   (progn (delete-region (line-beginning-position)
-                                         (line-end-position))
-                          (insert (cdr mapping)))
-                 (insert module)))
-             (haskell-mode-format-imports)))
-          (t
-           (let ((ident (save-excursion (forward-char -1) (haskell-ident-at-point))))
-             (insert " ")
-             (when ident
-               (haskell-process-do-try-info ident)))))))
+(defvar xref-prompt-for-identifier nil)
 
 ;;;###autoload
 (defun haskell-mode-jump-to-tag (&optional next-p)
-  "Jump to the tag of the given identifier."
+  "Jump to the tag of the given identifier.
+
+Give optional NEXT-P parameter to override value of
+`xref-prompt-for-identifier' during definition search."
   (interactive "P")
   (let ((ident (haskell-ident-at-point))
-        (tags-file-name (haskell-session-tags-filename (haskell-session)))
+        (tags-file-dir (haskell-cabal--find-tags-dir))
         (tags-revert-without-query t))
-    (when (and ident (not (string= "" (haskell-string-trim ident))))
-      (cond ((file-exists-p tags-file-name)
-             (let ((xref-prompt-for-identifier next-p))
-               (xref-find-definitions ident)))
-            (t (haskell-process-generate-tags ident))))))
+    (when (and ident
+               (not (string= "" (haskell-string-trim ident)))
+               tags-file-dir)
+      (let ((tags-file-name (concat tags-file-dir "TAGS")))
+        (cond ((file-exists-p tags-file-name)
+               (let ((xref-prompt-for-identifier next-p))
+                 (xref-find-definitions ident)))
+              (t (haskell-mode-generate-tags ident)))))))
 
 ;;;###autoload
 (defun haskell-mode-after-save-handler ()
   "Function that will be called after buffer's saving."
   (when haskell-tags-on-save
-    (ignore-errors (when (and (boundp 'haskell-session) haskell-session)
-                     (haskell-process-generate-tags))))
-  (when haskell-stylish-on-save
-    (ignore-errors (haskell-mode-stylish-buffer))
-    (let ((before-save-hook '())
-          (after-save-hook '()))
-      (basic-save-buffer))))
+    (ignore-errors (haskell-mode-generate-tags))))
 
 ;;;###autoload
-(defun haskell-mode-tag-find (&optional next-p)
+(defun haskell-mode-tag-find (&optional _next-p)
   "The tag find function, specific for the particular session."
   (interactive "P")
   (cond
@@ -360,12 +391,7 @@
   (interactive)
   (let* ((session (haskell-session))
          (buffer (haskell-session-interactive-buffer session)))
-    (unless (and (cl-find-if (lambda (window) (equal (window-buffer window) buffer))
-                             (window-list))
-                 (= 2 (length (window-list))))
-      (delete-other-windows)
-      (display-buffer buffer)
-      (other-window 1))))
+    (pop-to-buffer buffer)))
 
 ;;;###autoload
 (defun haskell-process-load-file ()
@@ -381,12 +407,18 @@
                                 (current-buffer)))
 
 ;;;###autoload
-(defun haskell-process-reload-file ()
+(defun haskell-process-reload ()
   "Re-load the current buffer file."
   (interactive)
   (save-buffer)
   (haskell-interactive-mode-reset-error (haskell-session))
-  (haskell-process-file-loadish "reload" t nil))
+  (haskell-process-file-loadish "reload" t (current-buffer)))
+
+;;;###autoload
+(defun haskell-process-reload-file () (haskell-process-reload))
+
+(make-obsolete 'haskell-process-reload-file 'haskell-process-reload
+               "2015-11-14")
 
 ;;;###autoload
 (defun haskell-process-load-or-reload (&optional toggle)
@@ -398,7 +430,10 @@
                       (if haskell-reload-p
                           "Now running :reload."
                         "Now running :load <buffer-filename>.")))
-    (if haskell-reload-p (haskell-process-reload-file) (haskell-process-load-file))))
+    (if haskell-reload-p (haskell-process-reload) (haskell-process-load-file))))
+
+(make-obsolete 'haskell-process-load-or-reload 'haskell-process-load-file
+               "2015-11-14")
 
 ;;;###autoload
 (defun haskell-process-cabal-build ()
@@ -420,9 +455,9 @@
                       (list "build --ghc-options=-fforce-recomp"))))))
 
 (defun haskell-process-file-loadish (command reload-p module-buffer)
-  "Run a loading-ish COMMAND that wants to pick up type errors
-and things like that. RELOAD-P indicates whether the notification
-should say 'reloaded' or 'loaded'. MODULE-BUFFER may be used
+  "Run a loading-ish COMMAND that wants to pick up type errors\
+and things like that.  RELOAD-P indicates whether the notification
+should say 'reloaded' or 'loaded'.  MODULE-BUFFER may be used
 for various things, but is optional."
   (let ((session (haskell-session)))
     (haskell-session-current-dir session)
@@ -464,7 +499,7 @@ for various things, but is optional."
   (insert-file-contents-literally
    (concat (haskell-session-current-dir (haskell-session))
            "/"
-           (haskell-guess-module-name)
+           (haskell-guess-module-name-from-file-name (buffer-file-name))
            ".imports")))
 
 (defun haskell-interactive-jump-to-error-line ()
@@ -495,3 +530,4 @@ for various things, but is optional."
                t))))))
 
 (provide 'haskell)
+;;; haskell.el ends here
