@@ -1,4 +1,4 @@
-;;; inf-haskell.el --- Interaction with an inferior Haskell process
+;;; inf-haskell.el --- Interaction with an inferior Haskell process -*- lexical-binding: t -*-
 
 ;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Free Software Foundation, Inc.
 
@@ -42,6 +42,7 @@
 ;; Dynamically scoped variables.
 (defvar find-tag-marker-ring)
 
+;;;###autoload
 (defgroup inferior-haskell nil
   "Settings for REPL interaction via `inferior-haskell-mode'"
   :link '(custom-manual "(haskell-mode)inferior-haskell-mode")
@@ -80,7 +81,7 @@ The command can include arguments."
   ;; The format of error messages used by Hugs.
   `(("^ERROR \"\\(.+?\\)\"\\(:\\| line \\)\\([0-9]+\\) - " 1 3)
     ;; Format of error messages used by GHCi.
-    ("^\\(.+?\\):\\([0-9]+\\):\\(\\([0-9]+\\):\\)?\\( \\|\n *\\)\\(Warning\\)?"
+    ("^\\(.+?\\):\\([0-9]+\\):\\(\\([0-9]+\\):\\)?\\( \\|\n *\\)\\([Ww]arning\\)?"
      1 2 4 ,@(if (fboundp 'compilation-fake-loc)
                  '((6) nil (5 '(face nil font-lock-multiline t)))))
     ;; Runtime exceptions, from ghci.
@@ -118,27 +119,26 @@ This will either look for a Cabal file or a \"module\" statement in the file."
 (define-derived-mode inferior-haskell-mode comint-mode "Inf-Haskell"
   "Major mode for interacting with an inferior Haskell process."
   :group 'inferior-haskell
-  (set (make-local-variable 'comint-prompt-regexp)
-       ;; Whay the backslash in [\\._[:alnum:]]?
-       "^\\*?[[:upper:]][\\._[:alnum:]]*\\(?: \\*?[[:upper:]][\\._[:alnum:]]*\\)*\\( λ\\)?> \\|^λ?> $")
-  (set (make-local-variable 'comint-input-autoexpand) nil)
+  (setq-local comint-prompt-regexp
+              ;; Why the backslash in [\\._[:alnum:]]?
+              "^\\*?[[:upper:]][\\._[:alnum:]]*\\(?: \\*?[[:upper:]][\\._[:alnum:]]*\\)*\\( λ\\)?> \\|^λ?> $")
+  (setq-local comint-input-autoexpand nil)
   (add-hook 'comint-preoutput-filter-functions
             'inferior-haskell-send-decl-post-filter)
   (add-hook 'comint-output-filter-functions 'inferior-haskell-spot-prompt nil t)
 
   ;; Setup directory tracking.
-  (set (make-local-variable 'shell-cd-regexp) ":cd")
+  (setq-local shell-cd-regexp ":cd")
   (condition-case nil
       (shell-dirtrack-mode 1)
     (error      ;The minor mode function may not exist or not accept an arg.
-     (set (make-local-variable 'shell-dirtrackp) t)
+     (setq-local shell-dirtrackp t)
      (add-hook 'comint-input-filter-functions 'shell-directory-tracker
                nil 'local)))
 
   ;; Setup `compile' support so you can just use C-x ` and friends.
-  (set (make-local-variable 'compilation-error-regexp-alist)
-       inferior-haskell-error-regexp-alist)
-  (set (make-local-variable 'compilation-first-column) 0) ;GHCI counts from 0.
+  (setq-local compilation-error-regexp-alist inferior-haskell-error-regexp-alist)
+  (setq-local compilation-first-column 0) ;GHCI counts from 0.
   (if (and (not (boundp 'minor-mode-overriding-map-alist))
            (fboundp 'compilation-shell-minor-mode))
       ;; If we can't remove compilation-minor-mode bindings, at least try to
@@ -209,14 +209,13 @@ setting up the inferior-haskell buffer."
   :type 'boolean
   :group 'inferior-haskell)
 
-(defvar inferior-haskell-send-decl-post-filter-on nil)
-(make-variable-buffer-local 'inferior-haskell-send-decl-post-filter-on)
+(defvar-local inferior-haskell-send-decl-post-filter-on nil)
 
 (defun inferior-haskell-send-decl-post-filter (string)
   (when (and inferior-haskell-send-decl-post-filter-on
-             #1=(string-match inferior-haskell-multiline-prompt-re string))
+             (string-match inferior-haskell-multiline-prompt-re string))
     ;; deleting sequence of `%s|' multiline promts
-    (while #1#
+    (while (string-match inferior-haskell-multiline-prompt-re string)
       (setq string (substring string (match-end 0))))
     ;; deleting regular prompts
     (setq string (replace-regexp-in-string comint-prompt-regexp "" string)
@@ -224,10 +223,9 @@ setting up the inferior-haskell buffer."
           inferior-haskell-send-decl-post-filter-on nil))
   string)
 
-(defvar inferior-haskell-seen-prompt nil)
-(make-variable-buffer-local 'inferior-haskell-seen-prompt)
+(defvar-local inferior-haskell-seen-prompt nil)
 
-(defun inferior-haskell-spot-prompt (string)
+(defun inferior-haskell-spot-prompt (_string)
   (let ((proc (get-buffer-process (current-buffer))))
     (when proc
       (save-excursion
@@ -259,8 +257,7 @@ The process PROC should be associated to a comint buffer."
                               ;; XEmacs needs this argument.
                               (current-buffer))
             inferior-haskell-cabal-buffer
-          (set (make-local-variable 'inferior-haskell-cabal-buffer)
-               (haskell-cabal-find-file))))))
+          (setq-local inferior-haskell-cabal-buffer (haskell-cabal-find-file))))))
 
 (defun inferior-haskell-find-project-root (buf)
   (with-current-buffer buf
@@ -271,7 +268,7 @@ The process PROC should be associated to a comint buffer."
            )
       (or (when cabal
             (with-current-buffer cabal
-              (let ((hsd (haskell-cabal-get-setting "hs-source-dirs")))
+              (let ((hsd (haskell-cabal--get-field "hs-source-dirs")))
                 (if (null hsd)
                     ;; If there's a Cabal file with no Hs-Source-Dirs, then
                     ;; just use the Cabal file's directory.
@@ -679,7 +676,6 @@ Insert the output into the current buffer."
       (message "Generating module alist... done")
       module-alist)))
 
-
 (defcustom inferior-haskell-module-alist-file
   ;; (expand-file-name "~/.inf-haskell-module-alist")
   (expand-file-name (concat "inf-haskell-module-alist-"
@@ -778,7 +774,7 @@ we load it."
   (let* (;; Find the module and look it up in the alist
          (module (inferior-haskell-get-module sym))
          (full-name (inferior-haskell-map-internal-ghc-ident (concat module "." sym)))
-         (success (string-match "\\(.*\\)\\.\\(.*\\)" full-name))
+         (_success (string-match "\\(.*\\)\\.\\(.*\\)" full-name))
          (module (match-string 1 full-name))
          (sym (match-string 2 full-name))
          (alist-record (assoc module (inferior-haskell-module-alist)))
@@ -797,26 +793,26 @@ we load it."
 
 (defvar inf-haskell-mode-map
   (let ((map (make-sparse-keymap)))
-    ;; (define-key map [?\M-C-x]     'inferior-haskell-send-defun)
-    ;; (define-key map [?\C-x ?\C-e] 'inferior-haskell-send-last-sexp)
-    ;; (define-key map [?\C-c ?\C-r] 'inferior-haskell-send-region)
-    (define-key map [?\C-x ?\C-d] 'inferior-haskell-send-decl)
-    (define-key map [?\C-c ?\C-z] 'switch-to-haskell)
-    (define-key map [?\C-c ?\C-l] 'inferior-haskell-load-file)
+    ;; (define-key map (kbd "M-C-x")    'inferior-haskell-send-defun)
+    ;; (define-key map (kbd "C-x C-e") 'inferior-haskell-send-last-sexp)
+    ;; (define-key map (kbd "C-c C-r") 'inferior-haskell-send-region)
+    (define-key map (kbd "C-x C-d") 'inferior-haskell-send-decl)
+    (define-key map (kbd "C-c C-z") 'switch-to-haskell)
+    (define-key map (kbd "C-c C-l") 'inferior-haskell-load-file)
     ;; I think it makes sense to bind inferior-haskell-load-and-run to C-c
     ;; C-r, but since it used to be bound to `reload' until June 2007, I'm
     ;; going to leave it out for now.
-    ;; (define-key map [?\C-c ?\C-r] 'inferior-haskell-load-and-run)
-    (define-key map [?\C-c ?\C-b] 'switch-to-haskell)
-    ;; (define-key map [?\C-c ?\C-s] 'inferior-haskell-start-process)
+    ;; (define-key map (kbd "C-c C-r") 'inferior-haskell-load-and-run)
+    (define-key map (kbd "C-c C-b") 'switch-to-haskell)
+    ;; (define-key map (kbd "C-c C-s") 'inferior-haskell-start-process)
     ;; That's what M-; is for.
     (define-key map (kbd "C-c C-t") 'inferior-haskell-type)
     (define-key map (kbd "C-c C-i") 'inferior-haskell-info)
     (define-key map (kbd "C-c M-.") 'inferior-haskell-find-definition)
     (define-key map (kbd "C-c C-d") 'inferior-haskell-find-haddock)
-    (define-key map [?\C-c ?\C-v] 'haskell-check)
+    (define-key map (kbd "C-c C-v") 'haskell-check)
     map)
-  "Keymap for using inf-haskell.")
+  "Keymap for using `inf-haskell-mode'.")
 
 ;;;###autoload
 (define-minor-mode inf-haskell-mode
